/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <aversive.h>
#include <aversive/queue.h>
#include <aversive/pgmspace.h>

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>

#include <parse.h>

#include "xbee_neighbor.h"
#include "xbee_atcmd.h"
#include "xbee_stats.h"
#include "xbee_buf.h"
#include "xbee_proto.h"
#include "xbee.h"

#include "parse_neighbor.h"

static int8_t
parse_neighbor(parse_pgm_token_hdr_t *tk, const char *buf, void *res)
{
	struct token_neighbor_data tkd;
	struct xbee_dev *dev;
	struct xbee_neigh *neigh;
	uint8_t token_len = 0;
	char bufcopy[32];

	memcpy_P(&tkd, &((struct token_neighbor *)tk)->neighbor_data,
		 sizeof(tkd));
	dev = *tkd.xbee_dev;

	while (!isendoftoken(buf[token_len]) &&
	       token_len < (sizeof(bufcopy)-1)) {
		bufcopy[token_len] = buf[token_len];
		token_len++;
	}
	bufcopy[token_len] = 0;
	neigh = xbee_neigh_lookup(dev, bufcopy);
	if (neigh == NULL) /* not found */
		return -1;

	/* store the address of xbee_neigh in structure */
	if (res)
		*(struct xbee_neigh **)res = neigh;

	return token_len;
}

static int8_t
complete_get_nb_neighbor(parse_pgm_token_hdr_t *tk)
{
	struct token_neighbor_data tkd;
	struct xbee_dev *dev;
	struct xbee_neigh *neigh;
	int8_t i = 0;

	memcpy_P(&tkd, &((struct token_neighbor *)tk)->neighbor_data,
		 sizeof(tkd));
	dev = *tkd.xbee_dev;

	LIST_FOREACH(neigh, &dev->neigh_list, next) {
		i++;
	}
	return i;
}

static int8_t
complete_get_elt_neighbor(parse_pgm_token_hdr_t *tk, int8_t idx,
			  char *dstbuf, uint8_t size)
{
	struct token_neighbor_data tkd;
	struct xbee_dev *dev;
	struct xbee_neigh *neigh;
	int8_t i = 0, len;

	memcpy_P(&tkd, &((struct token_neighbor *)tk)->neighbor_data,
		 sizeof(tkd));
	dev = *tkd.xbee_dev;

	LIST_FOREACH(neigh, &dev->neigh_list, next) {
		if (i++ == idx)
			break;
	}

	if (neigh == NULL)
		return -1;

	len = snprintf_P(dstbuf, size, PSTR("%s"), neigh->name);
	if (len < 0 || len >= size)
		return -1;

	return 0;
}


static int8_t
help_neighbor(parse_pgm_token_hdr_t *tk, char *dstbuf,
	      uint8_t size)
{
	snprintf_P(dstbuf, size, PSTR("Neighbor"));
	return 0;
}

struct token_ops token_neighbor_ops = {
	.parse = parse_neighbor,
	.complete_get_nb = complete_get_nb_neighbor,
	.complete_get_elt = complete_get_elt_neighbor,
	.get_help = help_neighbor,
};
